// $Id: padma.js,v 1.3.2.6 2005/06/17 21:14:24 vnagarjuna Exp $ -->

/* ***** BEGIN LICENSE BLOCK ***** 
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Nagarjuna Venna.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Nagarjuna Venna <vnagarjuna@yahoo.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//Browser related

function getTransformer(inputMethod, outputMethod, rtsStartMode)
{
    var transformer = Transformer.createTransformer(inputMethod, outputMethod);
    if (inputMethod == Transformer.method_RTS)
        transformer.setRTSMode(rtsStartMode);
    if (outputMethod == Transformer.method_RTS) {
        var prefObj = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
        var branch = prefObj.getBranch("Padma.");
        var RTSOutputFormat = branch.prefHasUserValue("RTSOutputFormat")  ? branch.getIntPref("RTSOutputFormat") : 0; //default = 0
        transformer.setRTSOutputFormat(RTSOutputFormat > 2 || RTSOutputFormat < 0 ? 0 : RTSOutputFormat);
    }

    return transformer;
}

//Based on input method, determine if the selected text needs to be transformed
function requiresTransform(transformer, node)
{
    var mode = transformer.getInputMethod();
    if (mode == Transformer.method_Unknown) {
        if (node.nodeValue.match(/[\u0C00-\u0C7F]/))
            return true;
    }
    if (mode == Transformer.method_DynFonts || mode == Transformer.method_Unknown) {
        for(var parent = node.parentNode; parent.nodeType != 9; parent = parent.parentNode) {
            if (parent.nodeType == 1 && parent.tagName == "FONT") {
                var face = parent.getAttribute("FACE");
                if (face != null)
                    return transformer.setDynamicFont(face);
            }
        }
        return false;
    }
    return true;
}

//Process all nodes with root "root"
function processNodes(transformer, selection, root, start, end, soff, eoff)
{
    if (root.nodeType == 3) {
        if (selection.containsNode(root, true) && requiresTransform(transformer, root)) {
            if (root == start && root == end)
                root.replaceData(soff, eoff - soff, transformer.convert(root.nodeValue.substring(soff, eoff)));
            else if (root == start)
                root.replaceData(soff, root.length - soff, transformer.convert(root.nodeValue.substring(soff)));
            else if (root == end)
                root.replaceData(0, eoff, transformer.convert(root.nodeValue.substring(0, eoff)));
            else root.replaceData(0, root.length, transformer.convert(root.nodeValue));
        }
    }
    else {
        for(var i = 0; i < root.childNodes.length; ++i) {
            processNodes(transformer, selection, root.childNodes.item(i), start, end, soff, eoff);
        }
    }
}

function traverseDOM(transformer, selection)
{
    for(var i = 0; i < selection.rangeCount; i++) {
        var range = selection.getRangeAt(i);
        processNodes(transformer, selection, range.commonAncestorContainer, range.startContainer, 
                    range.endContainer, range.startOffset, range.endOffset);
    }
}

//Used only for special controls
function setTransformedText(text)
{
    //The credit for this piece goes to Jed Brown of the BBCode extension.
    //http://jedbrown.net/1.0
    try {
        var command = "cmd_insertText";
        var controller = document.commandDispatcher.getControllerForCommand(command);
        
        if (controller && controller.isCommandEnabled(command)) {
            controller = controller.QueryInterface(Components.interfaces.nsICommandController);
            var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
            params = params.createInstance(Components.interfaces.nsICommandParams);
            
            params.setStringValue("state_data", text);
            controller.doCommandWithParams(command, params);
        }
    } catch (e) {
        dump("Can't do cmd_insertText! ");
        dump(e+"\n");
    }
}

//Callback
function onTransform(inputMethod, outputMethod, rtsStartMode)
{
    initializeRelationships();

    //Create a transformer
    var transformer = getTransformer(inputMethod, outputMethod, rtsStartMode);
    
    var node = document.popupNode;
    var nodeLocalName = node.localName.toUpperCase();

    var selection;

    //Check for Selection in various controls
    //This code is thanks to Jaap Haitsma of the dictionarySearch extension and roachfiend of a million extensions
    //http://roachfiend.com/
    //http://mozdev.org/pipermail/project_owners/2004-March/001833.html
    if ((nodeLocalName == "TEXTAREA") || (nodeLocalName == "INPUT" && (node.type == "text" || node.type == "password"))) {
        selection = node.value.substring(node.selectionStart, node.selectionEnd);
    } else if (nodeLocalName == "OPTION") {
        var parentSelect = node.parentNode;
        if (parentSelect.localName.toUpperCase() == "SELECT") {
            if (parentSelect.multiple) {
                var anOption;
                for (var i = 0; i < parentSelect.options.length; i++) {
                    anOption = parentSelect.options[i];
                    if (anOption.selected) {
                        selection += " " + anOption.value;
                    }
                }
            } else {
                selection = node.value;
            }
        }
    } else if (nodeLocalName == "SELECT") {
        selection = node.options[node.selected].value;
    }
    else {
        //Selection in a regular window
        var focusedWindow = document.commandDispatcher.focusedWindow;
        var selection = focusedWindow.getSelection();
        traverseDOM(transformer, selection);
        selection.collapseToEnd();
        return;
    }

    //For the specials
    setTransformedText(transformer.convert(selection));
}
