// $Id: parser.js,v 1.1.2.2 2005/06/16 22:49:43 vnagarjuna Exp $ -->

/* ***** BEGIN LICENSE BLOCK ***** 
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Nagarjuna Venna.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Nagarjuna Venna <vnagarjuna@yahoo.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//A very simple minded parser that returns one character at a time
//This is the prototype for more specialized parsers
//Also includes the state machine

//Parser states
Parser.state_START  = 0;    //Syllable hunt starts from this state
Parser.state_CONSNT = 1;    //Found a consonant to start the syllable
Parser.state_PREFIX = 2;    //Found a prefix symbol

function Parser(input) {
    this.index = 0;
    this.length = input.length;
    this.text = input;
    this.state = Parser.state_START;
}

Parser.prototype.more = function ()
{
    return this.index < this.length;
}

Parser.prototype.next = function ()
{
    return this.text[this.index++];
}

//Change symbol type based on state
//No need to change symbol type by default
Parser.prototype.updateSymbolType = function (type)
{
    return type;
}

Parser.prototype.handleConsonantTermination = function (current)
{
    return current;
}

Parser.prototype.isCurrentSyllableComplete = function (type)
{
    return type == Padma.type_spaccu || type == Padma.type_gunimtam;
}

//State Machine
//The parser has just determined that symbol is a match for key
//The syllable is built into current and is returned from this function
Parser.prototype.handleInput = function (symbol, key, current)
{
    var old_type = Padma.type_unknown;
    if (symbol != null)
        old_type = Padma.getType(symbol);
    else symbol = key;

    var prefix = false;
    if (arguments.length == 4) {
        prefix = arguments[3];
    }

    //alert("state = " + this.state);
    //alert("type = " + old_type);
        
    var type = this.updateSymbolType(old_type);
    if (type != old_type) {
        symbol = Padma.dependentForm(symbol);
    }
    
    switch (this.state) {
        case Parser.state_START:
        case Parser.state_PREFIX: {
            this.index += key.length;
            if (prefix == true) {
                this.state = Parser.state_PREFIX;
                return symbol;
            }
            if (type == Padma.type_accu || type == Padma.type_spaccu || type == Padma.type_digit || type == Padma.type_unknown) {
                //Current syllable is complete
                this.state = Parser.state_START;
                //alert("1: returning " + symbol);
            }
            else if (type == Padma.type_hallu) {
                //Wait for more
                this.state = Parser.state_CONSNT;
            }
            else {
                //alert("2: returning " + symbol);
                this.state = Parser.state_START;
            }
            return symbol;
        }
        
        case Parser.state_CONSNT: {
            if (type == Padma.type_accu || type == Padma.type_digit || type == Padma.type_unknown || type == Padma.type_hallu || prefix) {
                //next syllable has started, process it again next time around.
                //TODO: cache this symbol
                this.state = Parser.state_START;
                return this.handleConsonantTermination(current);
            }
            this.index += key.length;
            if (this.isCurrentSyllableComplete(type)) {
                //Syllable is complete
                this.state = Parser.state_START;
                //alert("5: returning " + current + symbol);
            }
            //vattu
            return current + symbol;
        }
    }
}
